#include <bits/stdc++.h>

const int ALPHABET = 3000 + 1;

struct hashed_array {
  const std::vector<int64_t> modulos = {static_cast<int>(1e9) + 7, 999999937};
  const std::vector<int64_t> bases = {239017, 239017};
  std::vector<std::vector<int64_t>> powers;
  std::vector<std::vector<int64_t>> hashes;

  int64_t mmod(int64_t x, int64_t mod) const {
    return (x % mod + mod) % mod;
  }

  hashed_array(const std::vector<int> &a) : 
    powers(modulos.size()),
    hashes(modulos.size()) {
    for (size_t i = 0; i < modulos.size(); i++) {
      int64_t modulo = modulos[i];
      int64_t base = bases[i];
      powers[i].resize(a.size() + 1);
      powers[i][0] = 1;
      for (size_t j = 1; j <= a.size(); j++) {
        powers[i][j] = mmod(powers[i][j - 1] * base, modulo);
      }
      hashes[i].resize(a.size() + 1);
      hashes[i][0] = 0;
      for (size_t j = 0; j < a.size(); j++) {
        hashes[i][j + 1] = mmod(hashes[i][j] * base + a[j], modulo);
      }
    }
  }

  int64_t get(int l, int r, int64_t modulo_id) const {
    return mmod(hashes[modulo_id][r + 1] - hashes[modulo_id][l] * powers[modulo_id][r - l + 1], modulos[modulo_id]);
  }

  std::vector<int64_t> get(int l, int r) const {
    std::vector<int64_t> result;
    for (size_t i = 0; i < modulos.size(); i++) {
      result.push_back(get(l, r, i));
    }
    return result;
  }

  int size() const {
    return static_cast<int>(powers[0].size()) - 1;
  }
};

std::vector<std::vector<int>> findMinSubsequence(const std::vector<int> &a) {
  int n = static_cast<int>(a.size());
  std::vector<std::vector<int>> first(n + 1, std::vector<int>(n));
  std::vector<std::vector<int>> firstAfter(n, std::vector<int>(ALPHABET, -1));
  for (int i = 0; i < n; i++) {
    for (int j = i; j < n; j++) {
      if (firstAfter[i][a[j]] == -1) {
        firstAfter[i][a[j]] = j;
      }
    }
  }
  for (int len = 1; len <= n; len++) {
    for (int i = 0; i < n; i++) {
      for (int c = 0; c < ALPHABET; c++) {
        int next = firstAfter[i][c];
        if (next != -1 && n - next >= len) {
          first[len][i] = next;
          break;
        }
      }
    }
  }
  std::vector<std::vector<int>> minSubsequence(n + 1);
  for (int len = 1; len <= n; len++) {
    int cur_len = len;
    int pos = 0;
    while (cur_len > 0) {
      minSubsequence[len].push_back(a[first[cur_len][pos]]);
      pos = first[cur_len][pos] + 1;
      cur_len--;
    }
  }
  return minSubsequence;
}

int maxEqualSubstring(const hashed_array &ha,
                      const hashed_array &hb,
                      int p1, int p2) {
  int l = 0;
  int r = std::min(ha.size() - p1, hb.size() - p2) + 1;
  while (l < r - 1) {
    int m = (l + r) / 2;
    if (ha.get(p1, p1 + m - 1) == hb.get(p2, p2 + m - 1)) {
      l = m;
    } else {
      r = m;
    }
  }
  return l;
}

std::vector<int> merge(const std::vector<int> &a, const std::vector<int> &b) {
  int n = static_cast<int>(a.size());
  int m = static_cast<int>(b.size());
  
  hashed_array ha(a);
  hashed_array hb(b);

  int p1 = 0;
  int p2 = 0;
  int p = 0;
  std::vector<int> res(n + m);
  while (p1 < n && p2 < m) {
    if (a[p1] < b[p2]) {
      res[p++] = a[p1++];
    } else if (a[p1] > b[p2]) {
      res[p++] = b[p2++];
    } else {
      int len = 1;
      if (p1 + len >= n) {
        res[p++] = b[p2++];
      } else if (p2 + len >= n) {
        res[p++] = a[p1++];
      } else if (a[p1 + len] < b[p2 + len]) {
        res[p++] = a[p1++];
      } else {
        res[p++] = b[p2++];
      }
    }
  }
  while (p1 < n) {
    res[p++] = a[p1++];
  }
  while (p2 < m) {
    res[p++] = b[p2++];
  }
  return res;
}

int main() {

  std::ios_base::sync_with_stdio(false);
  std::cin.tie(0);

  int n;
  std::cin >> n;
  std::vector<int> a(n);
  for (int i = 0; i < n; i++) {
    std::cin >> a[i];
  }
  int m;
  std::cin >> m;
  std::vector<int> b(m);
  for (int i = 0; i < m; i++) {
    std::cin >> b[i];
  }
  int k;
  std::cin >> k;
  std::vector<std::vector<std::vector<int>>> minSubsequence = {
    findMinSubsequence(a),
    findMinSubsequence(b)
  };
  std::vector<int> best;
  for (int alen = 1; alen < std::min(k, n + 1); alen++) {
    int blen = k - alen;
    if (blen > m) {
      continue;
    }
    std::vector<int> res = merge(minSubsequence[0][alen], minSubsequence[1][blen]);
    assert(static_cast<int>(res.size()) == k);
    if (best.size() == 0 || res < best) {
      best = res;
    }
  }
  for (int i = 0; i < k; i++) {
    std::cout << best[i] << ' ';
  }
  std::cout << '\n';
  return 0;
}
