#include <bits/stdc++.h>

const int ALPHABET = 3000 + 1;

struct hashed_array {
  const int64_t base = 239017;
  std::vector<int64_t> powers;
  std::vector<int64_t> hashes;

  hashed_array(const std::vector<int> &a) {
    powers.resize(a.size() + 1);
    powers[0] = 1;
    for (size_t j = 1; j <= a.size(); j++) {
      powers[j] = powers[j - 1] * base;
    }
    hashes.resize(a.size() + 1);
    hashes[0] = 0;
    for (size_t j = 0; j < a.size(); j++) {
      hashes[j + 1] = hashes[j] * base + a[j];
    }
  }

  int64_t get(int l, int r) const {
    return hashes[r + 1] - hashes[l] * powers[r - l + 1];
  }

  int size() const {
    return static_cast<int>(powers.size()) - 1;
  }
};

std::vector<std::vector<int>> find_min_subsequence(const std::vector<int> &a) {
  int n = static_cast<int>(a.size());
  std::vector<std::vector<int>> first(n + 1, std::vector<int>(n));
  std::vector<std::vector<int>> first_after(n, std::vector<int>(ALPHABET, -1));
  for (int i = 0; i < n; i++) {
    for (int j = i; j < n; j++) {
      if (first_after[i][a[j]] == -1) {
        first_after[i][a[j]] = j;
      }
    }
  }
  for (int i = 0; i < n; i++) {
    int last_maxlen = 0;
    for (int c = 0; c < ALPHABET; c++) {
      int next = first_after[i][c];
      if (next == -1) {
        continue;
      }
      int maxlen = n - next;
      for (int l = last_maxlen + 1; l <= maxlen; l++) {
        first[l][i] = next;
      }
      last_maxlen = std::max(last_maxlen, maxlen);
    }
  }
  std::vector<std::vector<int>> min_subsequence(n + 1);
  for (int len = 1; len <= n; len++) {
    int cur_len = len;
    int pos = 0;
    while (cur_len > 0) {
      min_subsequence[len].push_back(a[first[cur_len][pos]]);
      pos = first[cur_len][pos] + 1;
      cur_len--;
    }
  }
  return min_subsequence;
}

int max_equal_substring(const hashed_array &ha,
                      const hashed_array &hb,
                      int p1, int p2) {
  int l = 0;
  int r = std::min(ha.size() - p1, hb.size() - p2) + 1;
  while (l < r - 1) {
    int m = (l + r) / 2;
    if (ha.get(p1, p1 + m - 1) == hb.get(p2, p2 + m - 1)) {
      l = m;
    } else {
      r = m;
    }
  }
  return l;
}

std::vector<int> merge(const std::vector<int> &a, const std::vector<int> &b) {
  int n = static_cast<int>(a.size());
  int m = static_cast<int>(b.size());
  
  hashed_array ha(a);
  hashed_array hb(b);

  int p1 = 0;
  int p2 = 0;
  int p = 0;
  std::vector<int> res(n + m);
  while (p1 < n && p2 < m) {
    if (a[p1] < b[p2]) {
      res[p++] = a[p1++];
    } else if (a[p1] > b[p2]) {
      res[p++] = b[p2++];
    } else {
      int len = max_equal_substring(ha, hb, p1, p2);
      if (p1 + len >= n) {
        res[p++] = b[p2++];
      } else if (p2 + len >= m) {
        res[p++] = a[p1++];
      } else if (a[p1 + len] < b[p2 + len]) {
        res[p++] = a[p1++];
      } else {
        res[p++] = b[p2++];
      }
    }
  }
  while (p1 < n) {
    res[p++] = a[p1++];
  }
  while (p2 < m) {
    res[p++] = b[p2++];
  }
  return res;
}

int main() {

  std::ios_base::sync_with_stdio(false);
  std::cin.tie(0);

  int n;
  std::cin >> n;
  std::vector<int> a(n);
  for (int i = 0; i < n; i++) {
    std::cin >> a[i];
  }
  int m;
  std::cin >> m;
  std::vector<int> b(m);
  for (int i = 0; i < m; i++) {
    std::cin >> b[i];
  }
  int k;
  std::cin >> k;
  std::vector<std::vector<std::vector<int>>> min_subsequence = {
    find_min_subsequence(a),
    find_min_subsequence(b)
  };
  std::vector<int> best;
  for (int alen = 1; alen < std::min(k, n + 1); alen++) {
    int blen = k - alen;
    if (blen > m) {
      continue;
    }
    std::vector<int> res = merge(min_subsequence[0][alen], min_subsequence[1][blen]);
    assert(static_cast<int>(res.size()) == k);
    if (best.size() == 0 || res < best) {
      best = res;
    }
  }
  for (int i = 0; i < k; i++) {
    std::cout << best[i] << ' ';
  }
  std::cout << '\n';
  return 0;
}
